package org.apache.catalina.tribes.jmx;

import java.lang.management.ManagementFactory;
import java.util.Iterator;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.management.QueryExp;
import org.apache.catalina.tribes.Channel;
import org.apache.catalina.tribes.JmxChannel;
import org.apache.catalina.tribes.util.StringManager;
import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;

public class JmxRegistry {

   private static final Log log = LogFactory.getLog(JmxRegistry.class);
   protected static final StringManager sm = StringManager.getManager(JmxRegistry.class);
   private static ConcurrentHashMap registryCache = new ConcurrentHashMap();
   private MBeanServer mbserver = ManagementFactory.getPlatformMBeanServer();
   private ObjectName baseOname = null;


   public static JmxRegistry getRegistry(Channel channel) {
      if(channel != null && channel.getName() != null) {
         JmxRegistry registry = (JmxRegistry)registryCache.get(channel.getName());
         if(registry != null) {
            return registry;
         } else if(!(channel instanceof JmxChannel)) {
            return null;
         } else {
            JmxChannel jmxChannel = (JmxChannel)channel;
            if(!jmxChannel.isJmxEnabled()) {
               return null;
            } else {
               ObjectName baseOn = createBaseObjectName(jmxChannel.getJmxDomain(), jmxChannel.getJmxPrefix(), channel.getName());
               if(baseOn == null) {
                  return null;
               } else {
                  registry = new JmxRegistry();
                  registry.baseOname = baseOn;
                  registryCache.put(channel.getName(), registry);
                  return registry;
               }
            }
         }
      } else {
         return null;
      }
   }

   public static void removeRegistry(Channel channel, boolean clear) {
      JmxRegistry registry = (JmxRegistry)registryCache.get(channel.getName());
      if(registry != null) {
         if(clear) {
            registry.clearMBeans();
         }

         registryCache.remove(channel.getName());
      }
   }

   private static ObjectName createBaseObjectName(String domain, String prefix, String name) {
      if(domain == null) {
         log.warn(sm.getString("jmxRegistry.no.domain"));
         return null;
      } else {
         ObjectName on = null;
         StringBuilder sb = new StringBuilder(domain);
         sb.append(':');
         sb.append(prefix);
         sb.append("type=Channel,channel=");
         sb.append(name);

         try {
            on = new ObjectName(sb.toString());
         } catch (MalformedObjectNameException var6) {
            log.error(sm.getString("jmxRegistry.objectName.failed", new Object[]{sb.toString()}), var6);
         }

         return on;
      }
   }

   public ObjectName registerJmx(String keyprop, Object bean) {
      String oNameStr = this.baseOname.toString() + keyprop;
      ObjectName oName = null;

      try {
         oName = new ObjectName(oNameStr);
         if(this.mbserver.isRegistered(oName)) {
            this.mbserver.unregisterMBean(oName);
         }

         this.mbserver.registerMBean(bean, oName);
         return oName;
      } catch (NotCompliantMBeanException var6) {
         log.warn(sm.getString("jmxRegistry.registerJmx.notCompliant", new Object[]{bean}), var6);
         return null;
      } catch (MalformedObjectNameException var7) {
         log.error(sm.getString("jmxRegistry.objectName.failed", new Object[]{oNameStr}), var7);
         return null;
      } catch (Exception var8) {
         log.error(sm.getString("jmxRegistry.registerJmx.failed", new Object[]{bean, oNameStr}), var8);
         return null;
      }
   }

   public void unregisterJmx(ObjectName oname) {
      if(oname != null) {
         try {
            this.mbserver.unregisterMBean(oname);
         } catch (InstanceNotFoundException var3) {
            log.warn(sm.getString("jmxRegistry.unregisterJmx.notFound", new Object[]{oname}), var3);
         } catch (Exception var4) {
            log.warn(sm.getString("jmxRegistry.unregisterJmx.failed", new Object[]{oname}), var4);
         }

      }
   }

   private void clearMBeans() {
      String query = this.baseOname.toString() + ",*";

      try {
         ObjectName e = new ObjectName(query);
         Set onames = this.mbserver.queryNames(e, (QueryExp)null);
         Iterator var4 = onames.iterator();

         while(var4.hasNext()) {
            ObjectName objectName = (ObjectName)var4.next();
            this.unregisterJmx(objectName);
         }
      } catch (MalformedObjectNameException var6) {
         log.error(sm.getString("jmxRegistry.objectName.failed", new Object[]{query}), var6);
      }

   }

}
